@@ -1,19 +1,19 @@
 #! /usr/bin/env/python
 # -*- coding: utf-8 -*-
 
-## Copyright 1999-2011 by LivingLogic AG, Bayreuth/Germany
-## Copyright 1999-2011 by Walter Dörwald
+## Copyright 1999-2012 by LivingLogic AG, Bayreuth/Germany
+## Copyright 1999-2012 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
-## See ll/__init__.py for the license
+## See ll/xist/__init__.py for the license
 
 
-import sys, unittest, cStringIO
+import sys, unittest, io
 
 from xml.parsers import expat
 
-import py.test
+import pytest
 
 from ll import url
 from ll.xist import xsc, css, present, sims, xnd, xfind
@@ -28,38 +28,38 @@ present.reprtab = "  "
 
 def check_lenunicode(node, _len, content):
 	assert len(node) == _len
-	assert unicode(node) == content
+	assert str(node) == content
 
 
 def test_fraglen():
-	check_lenunicode(xsc.Frag(), 0, u"")
-	check_lenunicode(xsc.Frag(1), 1, u"1")
-	check_lenunicode(xsc.Frag(1, 2, 3), 3, u"123")
-	check_lenunicode(xsc.Frag(None), 0, u"")
-	check_lenunicode(xsc.Frag(None, None, None), 0, u"")
-	check_lenunicode(xsc.Frag(1, None, 2, None, 3, None, 4), 4, u"1234")
-	check_lenunicode(xsc.Frag(1, (2, 3)), 3, u"123")
-	check_lenunicode(xsc.Frag(1, (None, None)), 1, u"1")
+	check_lenunicode(xsc.Frag(), 0, "")
+	check_lenunicode(xsc.Frag(1), 1, "1")
+	check_lenunicode(xsc.Frag(1, 2, 3), 3, "123")
+	check_lenunicode(xsc.Frag(None), 0, "")
+	check_lenunicode(xsc.Frag(None, None, None), 0, "")
+	check_lenunicode(xsc.Frag(1, None, 2, None, 3, None, 4), 4, "1234")
+	check_lenunicode(xsc.Frag(1, (2, 3)), 3, "123")
+	check_lenunicode(xsc.Frag(1, (None, None)), 1, "1")
 
 
 def test_append():
 	for cls in (xsc.Frag, html.div):
 		node = cls()
 		node.append(1)
-		check_lenunicode(node, 1, u"1")
+		check_lenunicode(node, 1, "1")
 		node.append(2)
-		check_lenunicode(node, 2, u"12")
+		check_lenunicode(node, 2, "12")
 		node.append()
-		check_lenunicode(node, 2, u"12")
+		check_lenunicode(node, 2, "12")
 		node.append(3, 4)
-		check_lenunicode(node, 4, u"1234")
+		check_lenunicode(node, 4, "1234")
 		node.append(None)
-		check_lenunicode(node, 4, u"1234")
+		check_lenunicode(node, 4, "1234")
 		node.append((5, 6))
-		check_lenunicode(node, 6, u"123456")
+		check_lenunicode(node, 6, "123456")
 		node.append(html.p.Attrs.id(7))
-		check_lenunicode(node, 7, u"1234567")
-		with py.test.raises(TypeError):
+		check_lenunicode(node, 7, "1234567")
+		with pytest.raises(TypeError):
 			node.append(xml.Attrs(lang=8))
 
 
@@ -67,70 +67,70 @@ def test_extend():
 	for cls in (xsc.Frag, html.div):
 		node = cls()
 		node.extend([1])
-		check_lenunicode(node, 1, u"1")
+		check_lenunicode(node, 1, "1")
 		node.extend([2])
-		check_lenunicode(node, 2, u"12")
+		check_lenunicode(node, 2, "12")
 		node.extend([])
-		check_lenunicode(node, 2, u"12")
+		check_lenunicode(node, 2, "12")
 		node.extend([None])
-		check_lenunicode(node, 2, u"12")
+		check_lenunicode(node, 2, "12")
 		node.extend([3, 4])
-		check_lenunicode(node, 4, u"1234")
+		check_lenunicode(node, 4, "1234")
 		node.extend([[], [[], [5], []]])
-		check_lenunicode(node, 5, u"12345")
+		check_lenunicode(node, 5, "12345")
 
 
 def test_insert():
 	for cls in (xsc.Frag, html.div):
 		node = cls()
 		node.insert(0, 1)
-		check_lenunicode(node, 1, u"1")
+		check_lenunicode(node, 1, "1")
 		node.insert(0, 2)
-		check_lenunicode(node, 2, u"21")
+		check_lenunicode(node, 2, "21")
 		node.insert(0, 3, 4)
-		check_lenunicode(node, 4, u"3421")
+		check_lenunicode(node, 4, "3421")
 		node.insert(0, None)
-		check_lenunicode(node, 4, u"3421")
+		check_lenunicode(node, 4, "3421")
 		node.insert(0, (5, 6))
-		check_lenunicode(node, 6, u"563421")
+		check_lenunicode(node, 6, "563421")
 
 
 def test_iadd():
 	for cls in (xsc.Frag, html.div):
 		node = cls()
 		node += [1]
-		check_lenunicode(node, 1, u"1")
+		check_lenunicode(node, 1, "1")
 		node += [2]
-		check_lenunicode(node, 2, u"12")
+		check_lenunicode(node, 2, "12")
 		node += []
-		check_lenunicode(node, 2, u"12")
+		check_lenunicode(node, 2, "12")
 		node += [None]
-		check_lenunicode(node, 2, u"12")
+		check_lenunicode(node, 2, "12")
 		node += [3, 4]
-		check_lenunicode(node, 4, u"1234")
+		check_lenunicode(node, 4, "1234")
 		node += [[], [[], [5], []]]
-		check_lenunicode(node, 5, u"12345")
+		check_lenunicode(node, 5, "12345")
 
 
 def test_len():
 	for cls in (xsc.Frag, html.div):
-		check_lenunicode(cls(), 0, u"")
-		check_lenunicode(cls(1), 1, u"1")
-		check_lenunicode(cls(1, 2, 3), 3, u"123")
-		check_lenunicode(cls(None), 0, u"")
-		check_lenunicode(cls(None, None, None), 0, u"")
-		check_lenunicode(cls(1, None, 2, None, 3, None, 4), 4, u"1234")
-		check_lenunicode(cls(1, (2, 3)), 3, u"123")
-		check_lenunicode(cls(1, (None, None)), 1, u"1")
+		check_lenunicode(cls(), 0, "")
+		check_lenunicode(cls(1), 1, "1")
+		check_lenunicode(cls(1, 2, 3), 3, "123")
+		check_lenunicode(cls(None), 0, "")
+		check_lenunicode(cls(None, None, None), 0, "")
+		check_lenunicode(cls(1, None, 2, None, 3, None, 4), 4, "1234")
+		check_lenunicode(cls(1, (2, 3)), 3, "123")
+		check_lenunicode(cls(1, (None, None)), 1, "1")
 
 
 def test_standardmethods():
 	for node in common.allnodes():
 		node.compacted()
 		node.normalized()
-		list(node.walk((True, xfind.enterattrs, xfind.entercontent)))
-		list(node.walknodes((True, xfind.enterattrs, xfind.entercontent)))
-		list(node.walkpaths((True, xfind.enterattrs, xfind.entercontent)))
+		list(node.walk(entercontent=True, enterattrs=True, enterattr=None, enterelementnode=True, leaveelementnode=True, enterattrnode=True, leaveattrnode=True))
+		list(node.walknodes(entercontent=True, enterattrs=True, enterattr=None, enterelementnode=True, leaveelementnode=True, enterattrnode=True, leaveattrnode=True))
+		list(node.walkpaths(entercontent=True, enterattrs=True, enterattr=None, enterelementnode=True, leaveelementnode=True, enterattrnode=True, leaveattrnode=True))
 		node.pretty()
 		node.clone()
 		node.conv()
@@ -139,14 +139,13 @@ def test_standardmethods():
 
 def test_standardmethods2():
 	for node in (common.createelement(), common.createfrag()):
-		node.sorted()
 		node.shuffled()
 		node.reversed()
 
 
 def test_stringify():
 	for node in common.allnodes():
-		unicode(node)
+		str(node)
 		str(node)
 		node.string()
 		node.bytes()
@@ -163,7 +162,7 @@ def test_astext():
 def test_number():
 	node = html.div(class_=1234)
 	assert int(node["class_"]) == 1234
-	assert long(node["class_"]) == 1234L
+	assert int(node["class_"]) == 1234
 	assert abs(float(node["class_"]) - 1234.) < 1e-2
 	node = html.div(class_="1+1j")
 	compl = complex(node["class_"])
@@ -173,15 +172,15 @@ def test_number():
 
 def test_write():
 	node = html.div()
-	io = cStringIO.StringIO()
-	node.write(io, xhtml=2)
-	assert io.getvalue() == b"<div/>"
+	s = io.BytesIO()
+	node.write(s, xhtml=2)
+	assert s.getvalue() == b"<div/>"
 
 
 def test_mul():
-	node = xsc.Frag(u"a")
-	assert 3*node == xsc.Frag(list(u"aaa"))
-	assert node*3 == xsc.Frag(list(u"aaa"))
+	node = xsc.Frag("a")
+	assert 3*node == xsc.Frag(list("aaa"))
+	assert node*3 == xsc.Frag(list("aaa"))
 
 	node = html.div()
 	assert 3*node == xsc.Frag(html.div(), html.div(), html.div())
@@ -189,19 +188,19 @@ def test_mul():
 
 
 def test_text():
-	s = u"test"
+	s = "test"
 	node = xsc.Text(s)
 	hash(node)
 	assert len(node), 4
-	assert node[1] == xsc.Text(u"e")
+	assert node[1] == xsc.Text("e")
 	assert 3*node == xsc.Text(3*s)
 	assert node*3 == xsc.Text(s*3)
-	assert node[1:3] == xsc.Text(u"es")
-	assert node.capitalize() == xsc.Text(u"Test")
-	assert node.center(8) == xsc.Text(u"  test  ")
-	assert node.count(u"t") == 2
-	assert node.endswith(u"st") is True
-	assert node.index(u"s") == 2
+	assert node[1:3] == xsc.Text("es")
+	assert node.capitalize() == xsc.Text("Test")
+	assert node.center(8) == xsc.Text("  test  ")
+	assert node.count("t") == 2
+	assert node.endswith("st") is True
+	assert node.index("s") == 2
 	assert node.isalpha() is True
 	assert node.isalnum() is True
 	assert node.isdecimal() is False
@@ -211,39 +210,39 @@ def test_text():
 	assert node.isspace() is False
 	assert node.istitle() is False
 	assert node.isupper() is False
-	assert node.join(xsc.Frag(list(u"abc"))) == xsc.Frag(u"a", u"test", u"b", u"test", u"c")
-	assert node.ljust(6) == xsc.Text(u"test  ")
-	assert node.ljust(6, u".") == xsc.Text(u"test..")
-	assert node.lower() == xsc.Text(u"test")
-	assert xsc.Text(u"  test").lstrip() == xsc.Text(u"test")
-	assert node.replace(u"s", u"x") == xsc.Text(u"text")
-	assert node.rjust(6) == xsc.Text(u"  test")
-	assert node.rjust(6, u".") == xsc.Text(u"..test")
-	assert xsc.Text(u"test  ").rstrip() == xsc.Text(u"test")
-	assert node.rfind(u"s") == 2
-	assert node.rindex(u"s") == 2
-	assert node.split(u"e") == xsc.Frag(u"t", u"st")
-	assert xsc.Text(u"a\nb\n").splitlines() == xsc.Frag(u"a", u"b")
-	assert node.startswith(u"te") is True
-	assert xsc.Text(u"  test  ").strip() == xsc.Text(u"test")
-	assert node.swapcase() == xsc.Text(u"TEST")
-	assert node.title() == xsc.Text(u"Test")
-	assert node.upper() == xsc.Text(u"TEST")
+	assert node.join(xsc.Frag(list("abc"))) == xsc.Frag("a", "test", "b", "test", "c")
+	assert node.ljust(6) == xsc.Text("test  ")
+	assert node.ljust(6, ".") == xsc.Text("test..")
+	assert node.lower() == xsc.Text("test")
+	assert xsc.Text("  test").lstrip() == xsc.Text("test")
+	assert node.replace("s", "x") == xsc.Text("text")
+	assert node.rjust(6) == xsc.Text("  test")
+	assert node.rjust(6, ".") == xsc.Text("..test")
+	assert xsc.Text("test  ").rstrip() == xsc.Text("test")
+	assert node.rfind("s") == 2
+	assert node.rindex("s") == 2
+	assert node.split("e") == xsc.Frag("t", "st")
+	assert xsc.Text("a\nb\n").splitlines() == xsc.Frag("a", "b")
+	assert node.startswith("te") is True
+	assert xsc.Text("  test  ").strip() == xsc.Text("test")
+	assert node.swapcase() == xsc.Text("TEST")
+	assert node.title() == xsc.Text("Test")
+	assert node.upper() == xsc.Text("TEST")
 
 
 def test_charref():
 	node = chars.ouml()
 	hash(node)
 	assert len(node) == 1
-	assert node[0] == xsc.Text(u"ö")
-	assert 3*node == xsc.Text(u"ööö")
-	assert node*3 == xsc.Text(u"ööö")
+	assert node[0] == xsc.Text("ö")
+	assert 3*node == xsc.Text("ööö")
+	assert node*3 == xsc.Text("ööö")
 	assert node[1:-2] == xsc.Text()
-	assert node.capitalize() == xsc.Text(u"Ö")
-	assert node.center(5) == xsc.Text(u"  ö  ")
-	assert node.count(u"t") == 0
-	assert node.endswith(u"ö") is True
-	assert node.index(u"ö") == 0
+	assert node.capitalize() == xsc.Text("Ö")
+	assert node.center(5) == xsc.Text("  ö  ")
+	assert node.count("t") == 0
+	assert node.endswith("ö") is True
+	assert node.index("ö") == 0
 	assert node.isalpha() is True
 	assert node.isalnum() is True
 	assert node.isdecimal() is False
@@ -253,24 +252,24 @@ def test_charref():
 	assert node.isspace() is False
 	assert node.istitle() is False
 	assert node.isupper() is False
-	assert node.ljust(3) == xsc.Text(u"ö  ")
-	assert node.ljust(3, ".") == xsc.Text(u"ö..")
-	assert node.lower() == xsc.Text(u"ö")
-	assert node.replace(u"ö", "x") == xsc.Text("x")
-	assert node.rjust(3) == xsc.Text(u"  ö")
-	assert node.rjust(3, ".") == xsc.Text(u"..ö")
-	assert node.rfind(u"ö") == 0
-	assert node.rindex(u"ö") == 0
-	assert node.startswith(u"ö") is True
-	assert node.swapcase() == xsc.Text(u"Ö")
-	assert node.title() == xsc.Text(u"Ö")
-	assert node.upper() == xsc.Text(u"Ö")
+	assert node.ljust(3) == xsc.Text("ö  ")
+	assert node.ljust(3, ".") == xsc.Text("ö..")
+	assert node.lower() == xsc.Text("ö")
+	assert node.replace("ö", "x") == xsc.Text("x")
+	assert node.rjust(3) == xsc.Text("  ö")
+	assert node.rjust(3, ".") == xsc.Text("..ö")
+	assert node.rfind("ö") == 0
+	assert node.rindex("ö") == 0
+	assert node.startswith("ö") is True
+	assert node.swapcase() == xsc.Text("Ö")
+	assert node.title() == xsc.Text("Ö")
+	assert node.upper() == xsc.Text("Ö")
 
 
 def test_conv():
 	def mappedmapper(node, converter):
 		if isinstance(node, xsc.Text):
-			node = node.replace(u"gurk", u"hurz")
+			node = node.replace("gurk", "hurz")
 		return node
 
 	node = common.createfrag()
@@ -281,13 +280,13 @@ def test_conv():
 
 def test_repr():
 	tests = common.allnodes()
-	allpresenters = [c for c in present.__dict__.itervalues() if isinstance(c, type) and c is not present.Presenter and issubclass(c, present.Presenter)]
+	allpresenters = [c for c in present.__dict__.values() if isinstance(c, type) and c is not present.Presenter and issubclass(c, present.Presenter)]
 	for node in tests:
 		repr(node)
 		for class_ in allpresenters:
 			presenter = class_(node)
 			# do it multiple time, to make sure the presenter gets properly reset
-			for i in xrange(3):
+			for i in range(3):
 				list(presenter)
 				str(presenter)
 
@@ -296,18 +295,18 @@ def test_attrsclone():
 	class newa(html.a):
 		def convert(self, converter):
 			attrs = self.attrs.clone()
-			attrs[u"href"].insert(0, u"foo")
+			attrs["href"].insert(0, "foo")
 			e = html.a(self.content, attrs)
 			return e.convert(converter)
-	e = newa(u"gurk", href=u"hurz")
+	e = newa("gurk", href="hurz")
 	e = e.conv().conv()
-	assert unicode(e["href"]) == u"foohurz"
+	assert str(e["href"]) == "foohurz"
 
 
 def test_attributes():
-	node = html.h1(u"gurk", {xml.Attrs.lang: u"de"}, lang=u"de")
-	assert node.attrs.has(u"lang")
-	assert node.attrs.has_xml(u"lang")
+	node = html.h1("gurk", {xml.Attrs.lang: "de"}, lang="de")
+	assert node.attrs.has("lang")
+	assert node.attrs.has_xml("lang")
 
 	assert node.attrs.has(html.h1.Attrs.lang)
 	assert node.attrs.has_xml(html.h1.Attrs.lang)
@@ -315,7 +314,7 @@ def test_attributes():
 	assert node.attrs.has(xml.Attrs.lang)
 	assert node.attrs.has_xml(xml.Attrs.lang)
 
-	assert u"lang" in node.attrs
+	assert "lang" in node.attrs
 	assert html.h1.Attrs.lang in node.attrs
 	assert xml.Attrs.lang in node.attrs
 
@@ -333,7 +332,7 @@ def test_attributekeysvaluesitems():
 			res = list(node.attrs.values())
 			assert len(res) == 1
 			assert res[0].__class__ is node.Attrs.attr_
-			assert unicode(res[0]) == attrvalue
+			assert str(res[0]) == attrvalue
 		else:
 			res = list(node.attrs.values())
 			assert len(res) == 0
@@ -343,7 +342,7 @@ def test_attributekeysvaluesitems():
 			assert len(res) == 1
 			assert res[0][0] is attrclass
 			assert res[0][1].__class__ is attrclass
-			assert unicode(res[0][1]) == attrvalue
+			assert str(res[0][1]) == attrvalue
 		else:
 			res = list(node.attrs.items())
 			assert len(res) == 0
@@ -358,54 +357,54 @@ def test_attributekeysvaluesitems():
 			class attr_(xsc.TextAttr):
 				xmlname = "attr"
 
-	yield check, Test1(), Test1.Attrs.attr_, u"42"
-	yield check, Test1(attr_=17), Test1.Attrs.attr_, u"17"
+	yield check, Test1(), Test1.Attrs.attr_, "42"
+	yield check, Test1(attr_=17), Test1.Attrs.attr_, "17"
 	yield check, Test1(attr_=None), Test1.Attrs.attr_, None
 
 	yield check, Test2(), Test2.Attrs.attr_, None
-	yield check, Test2(attr_=17), Test2.Attrs.attr_, u"17"
+	yield check, Test2(attr_=17), Test2.Attrs.attr_, "17"
 	yield check, Test2(attr_=None), Test2.Attrs.attr_, None
 
 
 def test_attributeswithoutnames():
 	node = html.h1(
-		u"gurk",
-		{xml.Attrs.lang: u"de", xml.Attrs.base: u"http://www.livinglogic.de/"},
-		lang=u"de",
-		style=u"color: #fff",
-		align=u"right",
-		title=u"gurk",
-		class_=u"important",
+		"gurk",
+		{xml.Attrs.lang: "de", xml.Attrs.base: "http://www.livinglogic.de/"},
+		lang="de",
+		style="color: #fff",
+		align="right",
+		title="gurk",
+		class_="important",
 		id=42,
-		dir=u"ltr"
+		dir="ltr"
 	)
 	keys = set(node.attrs.keys())
 	keys.remove(html.h1.Attrs.class_)
 
-	keys1 = set(node.attrs.withoutnames(u"class_").keys())
+	keys1 = set(node.attrs.withoutnames("class_").keys())
 	assert keys == keys1
 
 	keys.remove(xml.Attrs.lang)
 	keys.remove(xml.Attrs.base)
-	keys2 = set(node.attrs.withoutnames(u"class_", xml.Attrs.lang, xml.Attrs.base).keys())
+	keys2 = set(node.attrs.withoutnames("class_", xml.Attrs.lang, xml.Attrs.base).keys())
 	assert keys == keys2
 
 	# Check that non existing attrs are handled correctly
-	keys3 = set(node.attrs.withoutnames(u"class_", u"src", xml.Attrs.lang, xml.Attrs.base).keys())
+	keys3 = set(node.attrs.withoutnames("class_", "src", xml.Attrs.lang, xml.Attrs.base).keys())
 	assert keys == keys3
 
 
 def test_attributeswithoutnames_xml():
 	node = html.h1(
-		u"gurk",
-		title=u"gurk",
-		class_=u"important",
+		"gurk",
+		title="gurk",
+		class_="important",
 		id=42,
 	)
 	keys = set(node.attrs.keys())
 	keys.remove(html.h1.Attrs.class_)
 
-	keys1 = set(node.attrs.withoutnames_xml(u"class").keys())
+	keys1 = set(node.attrs.withoutnames_xml("class").keys())
 	assert keys == keys1
 
 
@@ -416,51 +415,51 @@ def test_attributeswithnames():
 				default = 42
 
 	node = h1(
-		u"gurk",
-		{xml.Attrs.space: 1, xml.Attrs.lang: u"de"},
-		class_=u"gurk",
-		align=u"right"
+		"gurk",
+		{xml.Attrs.space: 1, xml.Attrs.lang: "de"},
+		class_="gurk",
+		align="right"
 	)
 
-	assert set(node.attrs.withnames(u"id").keys()) == set()
+	assert set(node.attrs.withnames("id").keys()) == set()
 
-	assert set(node.attrs.withnames(u"class_").keys()) == {html.h1.Attrs.class_}
+	assert set(node.attrs.withnames("class_").keys()) == {html.h1.Attrs.class_}
 
-	assert set(node.attrs.withnames(u"lang", u"align").keys()) == {h1.Attrs.lang, html.h1.Attrs.align}
+	assert set(node.attrs.withnames("lang", "align").keys()) == {h1.Attrs.lang, html.h1.Attrs.align}
 
-	assert set(node.attrs.withnames(h1.Attrs.lang, u"align").keys()) == {h1.Attrs.lang, html.h1.Attrs.align}
+	assert set(node.attrs.withnames(h1.Attrs.lang, "align").keys()) == {h1.Attrs.lang, html.h1.Attrs.align}
 
-	assert set(node.attrs.withnames(html.h1.Attrs.lang, u"align").keys()) == {h1.Attrs.lang, html.h1.Attrs.align}
+	assert set(node.attrs.withnames(html.h1.Attrs.lang, "align").keys()) == {h1.Attrs.lang, html.h1.Attrs.align}
 
 	node = html.h1(
-		u"gurk",
-		{xml.Attrs.space: 1, xml.Attrs.lang: u"de"},
-		lang=u"de",
-		class_=u"gurk",
-		align=u"right"
+		"gurk",
+		{xml.Attrs.space: 1, xml.Attrs.lang: "de"},
+		lang="de",
+		class_="gurk",
+		align="right"
 	)
 
-	assert set(node.attrs.withnames(u"id").keys()) == set()
+	assert set(node.attrs.withnames("id").keys()) == set()
 
-	assert set(node.attrs.withnames(u"class_").keys()) == {html.h1.Attrs.class_}
+	assert set(node.attrs.withnames("class_").keys()) == {html.h1.Attrs.class_}
 
-	assert set(node.attrs.withnames(u"lang", u"align").keys()) == {html.h1.Attrs.lang, html.h1.Attrs.align}
+	assert set(node.attrs.withnames("lang", "align").keys()) == {html.h1.Attrs.lang, html.h1.Attrs.align}
 
 	# no h1.Attrs.lang
-	assert set(node.attrs.withnames(h1.Attrs.lang, u"align").keys()) == {html.h1.Attrs.align}
+	assert set(node.attrs.withnames(h1.Attrs.lang, "align").keys()) == {html.h1.Attrs.align}
 
-	assert set(node.attrs.withnames(html.h1.Attrs.lang, u"align").keys()) == {html.h1.Attrs.lang, html.h1.Attrs.align}
+	assert set(node.attrs.withnames(html.h1.Attrs.lang, "align").keys()) == {html.h1.Attrs.lang, html.h1.Attrs.align}
 
 
 def test_attributeswithnames_xml():
 	node = html.h1(
-		u"gurk",
+		"gurk",
 		{xml.Attrs.space: 1},
-		lang=u"de",
-		class_=u"gurk",
-		align=u"right"
+		lang="de",
+		class_="gurk",
+		align="right"
 	)
-	assert set(node.attrs.withnames_xml(u"class").keys()) == {html.h1.Attrs.class_}
+	assert set(node.attrs.withnames_xml("class").keys()) == {html.h1.Attrs.class_}
 	assert set(node.attrs.withnames_xml(xml.Attrs.space).keys()) == {xml.Attrs.space}
 
 
@@ -471,12 +470,12 @@ def test_defaultattributes():
 			class withdef(xsc.TextAttr): default = 42
 			class withoutdef(xsc.TextAttr): pass
 	node = Test()
-	assert u"withdef" in node.attrs
-	assert u"withoutdef" not in node.attrs
-	with py.test.raises(xsc.IllegalAttrError):
-		u"illegal" in node.attrs
+	assert "withdef" in node.attrs
+	assert "withoutdef" not in node.attrs
+	with pytest.raises(xsc.IllegalAttrError):
+		"illegal" in node.attrs
 	node = Test(withdef=None)
-	assert u"withdef" not in node.attrs
+	assert "withdef" not in node.attrs
 
 
 def test_attributedictmethods():
@@ -500,15 +499,15 @@ def test_attributedictmethods():
 
 	check(
 		[ Test.Attrs.withdef, Test.Attrs.withoutdef ],
-		node.attrs.keys(),
+		list(node.attrs.keys()),
 	)
 	check(
 		[ Test.Attrs.withdef(42), Test.Attrs.withoutdef(42)],
-		node.attrs.values(),
+		list(node.attrs.values()),
 	)
 	check(
 		[ (Test.Attrs.withdef, Test.Attrs.withdef(42)), (Test.Attrs.withoutdef, Test.Attrs.withoutdef(42)) ],
-		node.attrs.items(),
+		list(node.attrs.items()),
 	)
 
 	check(
@@ -524,29 +523,29 @@ def test_fragattrdefault():
 				default = 42
 
 	node = testelem()
-	assert unicode(node[u"testattr"]) == u"42"
-	assert unicode(node.conv()[u"testattr"]) == u"42"
+	assert str(node["testattr"]) == "42"
+	assert str(node.conv()["testattr"]) == "42"
 
-	node[u"testattr"].clear()
-	assert u"testattr" not in node.attrs
-	assert u"testattr" not in node.conv().attrs
+	node["testattr"].clear()
+	assert "testattr" not in node.attrs
+	assert "testattr" not in node.conv().attrs
 
 	node = testelem(testattr=23)
-	assert unicode(node[u"testattr"]) == u"23"
-	assert unicode(node.conv()[u"testattr"]) == u"23"
+	assert str(node["testattr"]) == "23"
+	assert str(node.conv()["testattr"]) == "23"
 
-	del node[u"testattr"]
-	assert unicode(node[u"testattr"]) == u""
-	assert unicode(node.conv()[u"testattr"]) == u""
+	del node["testattr"]
+	assert str(node["testattr"]) == ""
+	assert str(node.conv()["testattr"]) == ""
 
-	node[u"testattr"] = 23
-	node[u"testattr"] = None
-	assert u"testattr" not in node.attrs
-	assert u"testattr" not in node.conv().attrs
+	node["testattr"] = 23
+	node["testattr"] = None
+	assert "testattr" not in node.attrs
+	assert "testattr" not in node.conv().attrs
 
 	node = testelem(testattr=None)
-	assert u"testattr" not in node.attrs
-	assert u"testattr" not in node.conv().attrs
+	assert "testattr" not in node.attrs
+	assert "testattr" not in node.conv().attrs
 
 
 def test_checkisallowed():
@@ -568,87 +567,87 @@ def test_checkisallowed():
 			testattr = None
 
 	node = testelem()
-	assert node.attrs.isallowed(u"testattr") is True
-	assert node.attrs.isallowed(u"notestattr") is False
+	assert node.attrs.isallowed("testattr") is True
+	assert node.attrs.isallowed("notestattr") is False
 
 	node = testelem2()
-	assert node.attrs.isallowed(u"testattr") is True
-	assert node.attrs.isallowed(u"notestattr") is False
+	assert node.attrs.isallowed("testattr") is True
+	assert node.attrs.isallowed("notestattr") is False
 
 	node = testelem3()
-	assert node.attrs.isallowed(u"testattr") is True
-	assert node.attrs.isallowed(u"testattr3") is True
+	assert node.attrs.isallowed("testattr") is True
+	assert node.attrs.isallowed("testattr3") is True
 
 	node = testelem4()
-	assert node.attrs.isallowed(u"testattr") is False
-	assert node.attrs.isallowed(u"testattr3") is True
+	assert node.attrs.isallowed("testattr") is False
+	assert node.attrs.isallowed("testattr3") is True
 
 
 def test_withsep():
 	for class_ in (xsc.Frag, html.div):
 		node = class_(1,2,3)
-		assert unicode(node.withsep(u",")) == u"1,2,3"
+		assert str(node.withsep(",")) == "1,2,3"
 		node = class_(1)
-		assert unicode(node.withsep(u",")) == u"1"
+		assert str(node.withsep(",")) == "1"
 		node = class_()
-		assert unicode(node.withsep(u",")) == u""
+		assert str(node.withsep(",")) == ""
 
 
 def test_allowedattr():
-	assert html.a.Attrs.allowedattr(u"href") is html.a.Attrs.href
-	with py.test.raises(xsc.IllegalAttrError):
-		html.a.Attrs.allowedattr(u"gurk")
+	assert html.a.Attrs.allowedattr("href") is html.a.Attrs.href
+	with pytest.raises(xsc.IllegalAttrError):
+		html.a.Attrs.allowedattr("gurk")
 	assert html.a.Attrs.allowedattr(xml.Attrs.lang) is xml.Attrs.lang
 
 	# Check inherited attributes
-	assert htmlspecials.plaintable.Attrs.allowedattr(u"border") is htmlspecials.plaintable.Attrs.border
+	assert htmlspecials.plaintable.Attrs.allowedattr("border") is htmlspecials.plaintable.Attrs.border
 	assert htmlspecials.plaintable.Attrs.allowedattr(htmlspecials.plaintable.Attrs.border) is htmlspecials.plaintable.Attrs.border
 	assert html.table.Attrs.allowedattr(htmlspecials.plaintable.Attrs.border) is html.table.Attrs.border
 
 
 def test_plaintableattrs():
 	e = htmlspecials.plaintable(border=3)
-	assert isinstance(e[u"border"], html.table.Attrs.border)
-	assert isinstance(e[u"cellpadding"], html.table.Attrs.cellpadding)
+	assert isinstance(e["border"], html.table.Attrs.border)
+	assert isinstance(e["cellpadding"], html.table.Attrs.cellpadding)
 	e = e.conv()
-	assert isinstance(e[u"border"], html.table.Attrs.border)
-	assert isinstance(e[u"cellpadding"], html.table.Attrs.cellpadding)
+	assert isinstance(e["border"], html.table.Attrs.border)
+	assert isinstance(e["cellpadding"], html.table.Attrs.cellpadding)
 
 
 def test_attrupdate():
-	node = html.a(href=u"gurk", class_=u"hurz")
-	node.attrs.update(xml.Attrs(lang=u"de"), {u"href": u"gurk2", html.a.Attrs.id: 42})
-	assert unicode(node[u"href"]) == u"gurk2"
-	assert unicode(node[u"id"]) == u"42"
-	assert unicode(node[xml.Attrs.lang]) == u"de"
+	node = html.a(href="gurk", class_="hurz")
+	node.attrs.update(xml.Attrs(lang="de"), {"href": "gurk2", html.a.Attrs.id: 42})
+	assert str(node["href"]) == "gurk2"
+	assert str(node["id"]) == "42"
+	assert str(node[xml.Attrs.lang]) == "de"
 
-	node = html.a({xml.Attrs.lang: u"de"}, href=u"gurk", class_=u"hurz")
-	assert unicode(node[xml.Attrs.lang]) == u"de"
+	node = html.a({xml.Attrs.lang: "de"}, href="gurk", class_="hurz")
+	assert str(node[xml.Attrs.lang]) == "de"
 
-	node = html.a(xml.Attrs(lang=u"de"), href=u"gurk", class_=u"hurz")
-	assert unicode(node[xml.Attrs.lang]) == u"de"
+	node = html.a(xml.Attrs(lang="de"), href="gurk", class_="hurz")
+	assert str(node[xml.Attrs.lang]) == "de"
 
 	class Gurk(xsc.Element):
 		model = False
 		class Attrs(xsc.Element.Attrs):
 			class gurk(xsc.TextAttr): pass
-			class hurz(xsc.TextAttr): default = u"hinz+kunz"
+			class hurz(xsc.TextAttr): default = "hinz+kunz"
 
 	node1 = Gurk()
 	node2 = Gurk(hurz=None)
 	node1.attrs.update(node2.attrs)
-	assert u"hurz" not in node1.attrs
+	assert "hurz" not in node1.attrs
 
 	node1 = Gurk(hurz=None)
 	node2 = Gurk()
 	node1.attrs.update(node2.attrs)
-	assert u"hurz" in node1.attrs
+	assert "hurz" in node1.attrs
 
 	node = Gurk(Gurk(hurz=None).attrs)
-	assert u"hurz" not in node.attrs
+	assert "hurz" not in node.attrs
 
 	attrs = Gurk.Attrs(Gurk.Attrs(hurz=None))
-	assert u"hurz" not in attrs
+	assert "hurz" not in attrs
 
 
 def test_classrepr():
@@ -665,21 +664,21 @@ def test_classrepr():
 
 def test_getitem():
 	for cls in (xsc.Frag, html.div):
-		e = cls(xrange(6))
+		e = cls(range(6))
 		# int
 		assert e[2] == xsc.Text(2)
 		assert e[-1] == xsc.Text(5)
 
 		# slice
-		assert e[:] == xsc.Frag(xrange(6))
-		assert e[:2] == xsc.Frag(0, 1)
-		assert e[-2:] == xsc.Frag(4, 5)
-		assert e[::2] == xsc.Frag(0, 2, 4)
-		assert e[1::2] == xsc.Frag(1, 3, 5)
-		assert e[::-1] == xsc.Frag(xrange(5, -1, -1))
+		assert e[:] == cls(range(6))
+		assert e[:2] == cls(0, 1)
+		assert e[-2:] == cls(4, 5)
+		assert e[::2] == cls(0, 2, 4)
+		assert e[1::2] == cls(1, 3, 5)
+		assert e[::-1] == cls(range(5, -1, -1))
 
 		# selector
-		e = cls((html.dt(i), html.dd(2*i)) for i in xrange(3))
+		e = cls((html.dt(i), html.dd(2*i)) for i in range(3))
 		assert xsc.Frag(e[html.dt]) == xsc.Frag(html.dt(0), html.dt(1), html.dt(2))
 		assert xsc.Frag(e[html.dt[1]]) == xsc.Frag(html.dt(1))
 		assert e[e[0]][0] is e[0] # selector for a single node (returns an iterator nevertheless)
@@ -689,20 +688,20 @@ def test_getitem():
 		assert xsc.Frag(e[e/html.dt]) == xsc.Frag(html.dt(0), html.dt(1), html.dt(2))
 		assert xsc.Frag(e[e.__class__/html.dt]) == xsc.Frag(html.dt(0), html.dt(1), html.dt(2))
 
-		for attr in (u"class_", xml.Attrs.lang):
-			e = cls(u"foo", html.div(u"bar", {attr: u"gurk"}), u"baz")
+		for attr in ("class_", xml.Attrs.lang):
+			e = cls("foo", html.div("bar", {attr: "gurk"}), "baz")
 			i = e[xsc.Text]
-			assert unicode(i.next()) == u"foo"
-			assert unicode(i.next()) == u"baz"
-			with py.test.raises(StopIteration):
-				i.next()
+			assert str(next(i)) == "foo"
+			assert str(next(i)) == "baz"
+			with pytest.raises(StopIteration):
+				next(i)
 
 		# list
-		for attr in (u"class_", xml.Attrs.lang):
-			node = cls(html.div(u"foo", html.div(u"bar", {attr: u"gurk"}), u"baz"))
-			assert node[[]] == node[:]
-			assert unicode(node[[0, 1]]) == u"bar"
-			assert unicode(node[[0, 1, attr]]) == u"gurk"
+		for attr in ("class_", xml.Attrs.lang):
+			node = cls(html.div("foo", html.div("bar", {attr: "gurk"}), "baz"))
+			assert node[[]] is node
+			assert str(node[[0, 1]]) == "bar"
+			assert str(node[[0, 1, attr]]) == "gurk"
 
 
 def test_setitem():
@@ -739,13 +738,13 @@ def test_setitem():
 		e[::-1] = range(6)
 		assert e == cls(range(5, -1, -1))
 
-		for attr in (u"class_", xml.Attrs.lang):
-			node = cls(html.div(u"foo", html.div({attr: u"gurk"}), u"bar"))
-			node[[0, 1, attr]] = u"hurz"
-			assert unicode(node[[0, 1, attr]]) == u"hurz"
-			with py.test.raises(ValueError):
+		for attr in ("class_", xml.Attrs.lang):
+			node = cls(html.div("foo", html.div({attr: "gurk"}), "bar"))
+			node[[0, 1, attr]] = "hurz"
+			assert str(node[[0, 1, attr]]) == "hurz"
+			with pytest.raises(ValueError):
 				node[[]] = None
-			with py.test.raises(ValueError):
+			with pytest.raises(ValueError):
 				del node[[]]
 
 
@@ -789,6 +788,14 @@ def test_delitem():
 		del e[1::2]
 		assert e == cls(0, 2, 4)
 
+		e = cls(range(6))
+		del e[lambda p:int(str(p[-1])) % 2]
+		assert e == cls(0, 2, 4)
+
+		e = cls(html.b(i) if i%2 else html.a(i) for i in range(6))
+		del e[html.b]
+		assert e == cls(html.a(0), html.a(2), html.a(4))
+
 
 def test_clone():
 	for cls in (xsc.Frag, html.div):
@@ -819,32 +826,26 @@ def test_clone():
 	e = html.div(id=(17, html.div(23), 42))
 	for src in (e, e.attrs):
 		dst = src.clone()
-		assert src[u"id"] is not dst[u"id"]
-		assert src[u"id"][0] is dst[u"id"][0]
-		assert src[u"id"][1] is not dst[u"id"][1]
+		assert src["id"] is not dst["id"]
+		assert src["id"][0] is dst["id"][0]
+		assert src["id"][1] is not dst["id"][1]
 
-	e[u"id"][1] = e # create a cycle
-	e[u"id"][2] = e # create a cycle
+	e["id"][1] = e # create a cycle
+	e["id"][2] = e # create a cycle
 	for src in (e, e.attrs):
 		dst = src.copy()
-		assert src[u"id"] is dst[u"id"]
-		assert src[u"id"][0] is dst[u"id"][0]
-		assert src[u"id"][1] is dst[u"id"][1]
-		assert dst[u"id"][1] is dst[u"id"][2]
+		assert src["id"] is dst["id"]
+		assert src["id"][0] is dst["id"][0]
+		assert src["id"][1] is dst["id"][1]
+		assert dst["id"][1] is dst["id"][2]
 		dst = src.deepcopy()
-		assert src[u"id"] is not dst[u"id"]
-		assert src[u"id"][0] is dst[u"id"][0]
-		assert src[u"id"][1] is not dst[u"id"][1]
-		assert dst[u"id"][1] is dst[u"id"][2]
+		assert src["id"] is not dst["id"]
+		assert src["id"][0] is dst["id"][0]
+		assert src["id"][1] is not dst["id"][1]
+		assert dst["id"][1] is dst["id"][2]
 
 
-def test_sortedreversed():
-	for class_ in (xsc.Frag, html.div):
-		node = class_(3, 2, 1)
-		node2 = node.sorted(key=unicode)
-		assert node == class_(3, 2, 1)
-		assert node2 == class_(1, 2, 3)
-
+def test_reversed():
 	for class_ in (xsc.Frag, html.div):
 		node = class_(3, 2, 1)
 		node2 = node.reversed()
@@ -874,8 +875,8 @@ def test_with():
 
 	with xsc.build():
 		with html.p() as e:
-			+xml.Attrs(lang=u"de")
-	assert e == html.p(xml.Attrs(lang=u"de"))
+			+xml.Attrs(lang="de")
+	assert e == html.p(xml.Attrs(lang="de"))
 	assert e.bytes() == b'<p xml:lang="de"></p>'
 
 	with xsc.build():
@@ -891,29 +892,29 @@ def test_with():
 
 	with xsc.build():
 		with html.p() as e:
-			xsc.add(class_=u"foo")
-	assert e == html.p(class_=u"foo")
+			xsc.add(class_="foo")
+	assert e == html.p(class_="foo")
 
 	with xsc.build():
 		with html.p() as e:
-			xsc.add(dict(class_=u"foo"))
-	assert e == html.p(class_=u"foo")
+			xsc.add(dict(class_="foo"))
+	assert e == html.p(class_="foo")
 
 	with xsc.build():
 		with html.p() as e:
-			xsc.add(xml.Attrs(lang=u"en"))
-	assert e == html.p(xml.Attrs(lang=u"en"))
+			xsc.add(xml.Attrs(lang="en"))
+	assert e == html.p(xml.Attrs(lang="en"))
 
 
 def test_with_addattr():
 	with xsc.build():
 		with html.ul() as e:
-			with xsc.addattr(u"id"):
-				+xsc.Text(u"gurk")
-	assert e == html.ul(id=u"gurk")
+			with xsc.addattr("id"):
+				+xsc.Text("gurk")
+	assert e == html.ul(id="gurk")
 
 	with xsc.build():
 		with html.ul() as e:
 			with xsc.addattr(html.ul.Attrs.id):
-				+xsc.Text(u"gurk")
-	assert e == html.ul(id=u"gurk")
+				+xsc.Text("gurk")
+	assert e == html.ul(id="gurk")
